//
//  CommentQueryFilterView.swift
//  SampleApp
//
//  Created by Nishan Niraula on 20/2/2566 BE.
//  Copyright © 2566 BE David Zhang. All rights reserved.
//

import SwiftUI
import AmitySDK

struct CommentQueryFilterView: View {

    @Environment(\.presentationMode) var presentationMode
    
    let orderBy: AmityOrderBy
    let dataTypeFilter: AmityCommentDataTypeFilter?
    let includeDeleted: Bool
    let filterAction: ((AmityOrderBy, AmityCommentDataTypeFilter?, Bool) -> Void)?
    
    private let sortOptions = [CommentOrderByOption.lastCreated.rawValue,
                               CommentOrderByOption.firstCreated.rawValue]
    
    private let queryTypesFilterOptions = [CommentQueryFilterOption.any.rawValue,
                                           CommentQueryFilterOption.exact.rawValue]
    
    private let queryTypesOptions = [CommentDataTypeOption.text.rawValue,
                                     CommentDataTypeOption.image.rawValue,
                                     CommentDataTypeOption.imageAndText.rawValue]
    
    private let includeDeletedOptions = [IncludeDeletedOption.includeDeleted.rawValue,
                                         IncludeDeletedOption.excludeDeleted.rawValue]
    
    @State private var selectedSortOption = ""
    @State private var selectedQueryTypeFilterOption = ""
    @State private var selectedQueryTypesOption = ""
    @State private var selectedIncludeDeletedOption = ""
    
    var body: some View {
        VStack(alignment: .leading) {
            Text("Order By:")
                .font(.system(size: 16, weight: .semibold))
                .padding(.top, 16)
            
            Picker("Order By", selection: $selectedSortOption) {
                ForEach(sortOptions, id: \.self) {
                    Text($0)
                }
            }.pickerStyle(.segmented)
            
            Text("Query Types:")
                .padding(.top, 16)
                .font(.system(size: 16, weight: .semibold))
            Picker("Query Types", selection: $selectedQueryTypeFilterOption) {
                ForEach(queryTypesFilterOptions, id: \.self) {
                    Text($0)
                }
            }.pickerStyle(.segmented)
            
            Picker("Data Type", selection: $selectedQueryTypesOption) {
                ForEach(queryTypesOptions, id: \.self) {
                    Text($0)
                }
            }.pickerStyle(.segmented)
            
            Text("Delete Option")
                .padding(.top, 16)
                .font(.system(size: 16, weight: .semibold))
            Picker("Include Deleted Option", selection: $selectedIncludeDeletedOption) {
                ForEach(includeDeletedOptions, id: \.self) {
                    Text($0)
                }
            }.pickerStyle(.segmented)
            
            HStack {
                Spacer()
                
                Button {
                    let orderOption = CommentOrderByOption(rawValue: selectedSortOption) ?? .lastCreated
                    
                    let dataTypeOption = CommentDataTypeOption(rawValue: selectedQueryTypesOption) ?? .imageAndText
                    
                    let includeDeletedOption = IncludeDeletedOption(rawValue: selectedIncludeDeletedOption) ?? .excludeDeleted
                    
                    let queryTypeOptions: CommentQueryFilterOption? = CommentQueryFilterOption(rawValue: selectedQueryTypeFilterOption) ?? nil
                    
                    var queryFilter: AmityCommentDataTypeFilter?
                    if let queryTypeOptions {
                        switch queryTypeOptions {
                        case .exact:
                            queryFilter = .exact(dataTypeOption.value)
                        case .any:
                            queryFilter = .any(dataTypeOption.value)
                        }
                    }
                    
                    // Notify about applied filter
                    filterAction?(orderOption.value, queryFilter, includeDeletedOption.value)
                    
                    // Dismiss this screen
                    presentationMode.wrappedValue.dismiss()
                } label: {
                    Text("Apply")
                        .font(.headline)
                        .padding(.horizontal, 32)
                        .padding(.vertical, 6)
                }
                .buttonStyle(.borderedProminent)

                Spacer()
            }
            .padding(.top, 24)
            
            Spacer()
        }
        .padding(.horizontal)
        .navigationTitle("Query Options")
        .navigationBarTitleDisplayMode(.inline)
        .task {
            selectedSortOption = orderBy == .descending ? CommentOrderByOption.lastCreated.rawValue : CommentOrderByOption.firstCreated.rawValue
            
            selectedIncludeDeletedOption = includeDeleted ? IncludeDeletedOption.includeDeleted.rawValue : IncludeDeletedOption.excludeDeleted.rawValue
            
            let computeQueryOptions: ([AmityCommentDataType]) -> Void = { dataTypes in
                if dataTypes.contains(.image) && dataTypes.contains(.text) {
                    selectedQueryTypesOption = CommentDataTypeOption.imageAndText.rawValue
                } else if dataTypes.contains(.text) {
                    selectedQueryTypesOption = CommentDataTypeOption.text.rawValue
                } else if dataTypes.contains(.image) {
                    selectedQueryTypesOption = CommentDataTypeOption.image.rawValue
                }
            }
            
            if let dataTypeFilter {
                switch dataTypeFilter {
                case .any(let dataTypes):
                    selectedQueryTypeFilterOption = CommentQueryFilterOption.any.rawValue
                    computeQueryOptions(dataTypes)
                case .exact(let dataTypes):
                    selectedQueryTypeFilterOption = CommentQueryFilterOption.exact.rawValue
                    computeQueryOptions(dataTypes)
                default:
                    break
                }
            }
        }
    }
}

struct CommentQueryFilterView_Previews: PreviewProvider {
    static var previews: some View {
        CommentQueryFilterView(orderBy: .descending, dataTypeFilter: nil, includeDeleted: false, filterAction: nil)
    }
}

enum CommentOrderByOption: String {
    case lastCreated = "Last Created"
    case firstCreated = "First Created"
    
    var value: AmityOrderBy {
        switch self {
        case .lastCreated:
            return .descending
        case .firstCreated:
            return .ascending
        }
    }
}

enum IncludeDeletedOption: String {
    case includeDeleted = "Include Deleted"
    case excludeDeleted = "Exclude Deleted"
    
    var value: Bool {
        switch self {
        case .includeDeleted:
            return true
        case .excludeDeleted:
            return false
        }
    }
}

enum CommentDataTypeOption: String {
    case text = "Text"
    case image = "Image"
    case imageAndText = "Image & Text"
    
    var value: [AmityCommentDataType] {
        switch self {
        case .text:
            return [.text]
        case .image:
            return [.image]
        case .imageAndText:
            return [.image, .text]
        }
    }
}

enum CommentQueryFilterOption: String {
    case exact = "Exact"
    case any = "Any"
}
